% Example demonstrating the use of the RoundCut for segmenting a single
% surface using two-step refinement.
% Some of the visualization is commented out, but has not been removed 
% from the script.

clear, close all
addpath 'round_cut' 'GraphCut' 'mesh_functions'...
    'visualization_functions' 'test_cases'

[cost,ground_truth] = citrus_press_surface;

% VISUALIZATION OF GROUND TRUTH
figure, h2 = patch(ground_truth);
set(h2,'EdgeColor','none','FaceColor','r','FaceLighting','phong')
axis equal off, view(3), camlight right
title(['ground truth, |V|=',num2str(size(ground_truth.vertices,1))])

%% INITIAL CUT
nu = 5; % initial sphere frequency (icosahedron subdivision step)
sphere = sphere_tiling(nu); % constructing initial polyhedron
sphere = translate_mesh(sphere,(size(cost)+1)/2); % moving sphere to the volume center
q = 1:0.5*norm(size(cost)); % sampling from center to the volume boundary
U = sample_volume(cost,sphere,q,1); % sampling along sphere normals

delta_q = 10; % inter-column shift (smoothness parameter)
s = optimal_cut(U,sphere.adjacency,delta_q); % finding optimal cut
S = fold_surface(s,sphere,q); % computing new vertex positions
result0 = change_geometry(sphere,S,true); % displacing mesh vertices

%figure, imagesc(U), colormap gray, hold on, plot(1:numel(s),s,'r')
%title('initial cut in unfolded space')
figure, animate_volume_and_contours(cost,result0,0,'xyz','nonempty')

figure, h = draw_mesh(result0);
set(h,'EdgeColor','none','FaceColor','r','FaceLighting','phong')
axis equal off, view(3), camlight right
title(['initial cut, |V|=',num2str(size(result0.vertices,1))])

%% REFINEMENT CUT

subdivision_iter = 2; % number of subdivision steps
sub1 = mesh_subdivision(result0,subdivision_iter,'butterfly'); % subdividing
q1 = -5:0.5:5; % sampling along vertex normals both inside and outside the mesh
U1 = sample_volume(cost,sub1,q1,1); % sampling
delta_q1 = 20; % inter-column shift (smoothness parameter)

s1 = optimal_cut(U1,sub1.adjacency,delta_q1);
S1 = fold_surface(s1,sub1,q1);
result1 = change_geometry(sub1,S1,true);

%figure, imagesc(U1), colormap gray, hold on, plot(1:numel(s1),s1,'r')
%title('refinemant cut in unfolded space')
figure, animate_volume_and_contours(cost,result1,0,'xyz','nonempty')

figure ,h = draw_mesh(result1);
set(h,'EdgeColor','none','FaceColor','r','FaceLighting','phong')
axis equal off, view(3), camlight right
title(['refinement cut, |V|=',num2str(size(result1.vertices,1))])
