% Example demonstrating the use of the RoundCut for for iterative
% refinement. All intermediate results are shown, so running the script
% produces 50 matlab figures.

addpath 'round_cut' 'GraphCut' 'mesh_functions' ...
    'visualization_functions' 'cost_functions' 'test_cases'
clear, close all

V = bended_pipe;
V = V(21:end-20,51:end-50,61:end-40);

cost_in = cost_R_normalized_probability(V,[190 90],[10 20]);

%% %% INITIALIZATION %% %%

nu = 2; % INPUT
sphere = sphere_tiling(nu);
sphere.vertices = 10*sphere.vertices;

r = 2; % INPUT
q = 0:1:50;
%sphere = translate_mesh(sphere,[50.5 80.5 70.5]); % centered
sphere = translate_mesh(sphere,[60.5 50.5 60.5]); % off center

figure, draw_mesh_and_slice_bended(V,sphere)
%%
U = sample_volume(cost_in,sphere,q,[1 0]);
delta_q = 10; % INPUT (in units of r)
s = optimal_k_cut([],U,sphere.adjacency,delta_q,[]);

S = fold_surface(s,sphere,q);
result = change_geometry(sphere,S,true);

figure, imagesc(U(:,:,1)), colormap gray, hold on, plot(1:numel(s),s,'r')
figure, draw_mesh_and_slice_bended(V,result)


%% %% SUBDIVISION ITERATIONS %% %%

step_iter = 15; % INPUT
subdivision_mod = 4; % INPUT
subdivision_iter = 1; % INPUT

for n = 1:step_iter
    disp(['iteration ',num2str(n),'/',num2str(step_iter)])
    
    if mod(n,subdivision_mod)==0
        result = mesh_subdivision(result,subdivision_iter,'butterfly');
    end
    
    q = -15:1:15; % INPUT
    
    U = sample_volume(cost_in,result,q,[1 0]);
    
    delta_q = 20; % INPUT
    s = optimal_k_cut([],U,result.adjacency,delta_q,[]);
    
    S = fold_surface(s,result,q);
    result = change_geometry(result,S,true);
    
    figure, imagesc(U(:,:,1)), colormap gray, hold on, plot(1:numel(s),s,'r')
    figure, draw_mesh_and_slice_bended(V,result)
    
    % %% SMOOTHING %% %%
    lambda = 0.5; % INPUT
    result = explicit_taubin_smoothing(result,lambda,[],20);
    figure, draw_mesh_and_slice_bended(V,result)
    
end

%% %% REFINEMENT %% %%

q = -5:0.1:5; % INPUT
U = sample_volume(cost_in,result,q,[1 0]);

delta_q = 10; % INPUT
s = optimal_k_cut([],U,result.adjacency,delta_q,[]);

S = fold_surface(s,result,q);
result = change_geometry(result,S,true);

figure, draw_mesh_and_slice_bended(V,result)    



