function U = sample_volume(V,mesh,q,padding)
% V - a 3D matrix of cost volume.
% mesh - a triangle mesh structure defining sampling rays.
% Required fields: mesh.vertices and mesh.normals
% q - a vector defining sampling points allong each ray.
% U - resulting cost image.
% padding
% NOTE: Uses interp3 which assumes meshgrid format
%
% Author: Vedrana Andersen Dahl, vand at dtu.dk
% Date: 3.7.2013.

tic, fprintf('Sample volume ... ')

if nargin<4
    padding = 0;
end

X = ones(numel(q),1)*mesh.vertices(:,1)' + q(:)*mesh.normals(:,1)';
Y = ones(numel(q),1)*mesh.vertices(:,2)' + q(:)*mesh.normals(:,2)';
Z = ones(numel(q),1)*mesh.vertices(:,3)' + q(:)*mesh.normals(:,3)';

%figure, plot3(X,Y,Z) % showing sampling spikes

% sampling V
V = double(V);
U = zeros(length(q),size(mesh.vertices,1),size(V,4));
for k = 1:size(V,4)
    % pre-R2013a will pad with NaN for linear
    % If I was sure that version above R2013a is used I could (probably)
    % use linear extrapolation and remove padding
    if verLessThan('matlab','8.1')
        F = griddedInterpolant(V(:,:,:,k),'linear'); 
    else
        F = griddedInterpolant(V(:,:,:,k),'linear','none'); 
    end
    Uk = F(X,Y,Z);
    Uk(isnan(Uk)) = padding(k);
    U(:,:,k) = Uk;
    %U(:,:,k) = interp3(V(:,:,:,k),X,Y,Z,'linear',padding(k));
end

disp(['finished at ',num2str(toc)])

end
