function [V,ground_truth1,ground_truth2] = citrus_press_regions
% makes a test volume containing three regions (two surfaces)
% returnes also ground truth surfaces

dim = [200 200 200];

[X,Y,Z] = ndgrid(1:dim(1),1:dim(2),1:dim(3));

% making rotation matrix
u = [1 2 0];
u = u/norm(u); % rotation axis
alpha = pi/3; % rotation angle
U = zeros(3); % cross product matrix
U([2 6 7])  = u([3 1 2]);
U([3 4 8])  = -u([2 3 1]);
R = eye(3)*cos(alpha) + sin(alpha)*U + (1-cos(alpha))*kron(u',u);
c = (dim+1)/2;

% rotating
P = [X(:)-c(1) Y(:)-c(2) Z(:)-c(3)]';
PR = R*P;
X(:) = PR(1,:)+c(1);
Y(:) = PR(2,:)+c(2);
Z(:) = PR(3,:)+c(3);

% making the distance field
phi = atan2(Y-c(2),X-c(1)); % in [-pi,pi]
theta = acos((Z-c(3))./((X-c(1)).^2+(Y-c(2)).^2+(Z-c(3)).^2).^0.5); % in [0,pi]
r1 = min(dim)/4*... basic radius
    (1+0.2*sin(8*phi-0.1*Z).*... slices
    (1-cos(2*theta))/2); % spiraling
r2 = min(dim)/4*... basic radius
    (1.16+0.06*sin(8*phi-0.1*Z).*... slices
    (1-cos(2*theta))/2); % spiraling
V1 = (X-c(1)).^2+(Y-c(2)).^2+(Z-c(3)).^2 - r1.^2;
V2 = (X-c(1)).^2+(Y-c(2)).^2+(Z-c(3)).^2 - r2.^2;

% three regions: in, between and out
in = V1<=0;
between = V1>0 & V2<0;
out = V2>=0;

R = randn(size(V1));
mu = [0.6 0.9 0.1];
sigma2 = [0.01 0.02 0.001];

V = (mu(1)+sigma2(1)*R).*in + (mu(2)+sigma2(2)*R).*between...
    + (mu(3)+sigma2(3)*R).*out;

% cutting extreme values
V(V<0)=0;
V(V>1)=1;

% zero surfaces
ground_truth1 = isosurface(permute(V1,[2 1 3]),0);
ground_truth2 = isosurface(permute(V2,[2 1 3]),0);

% visualization
%figure, colormap gray, animate_volume(V,0,'xyz')
%p1 = patch(ground_truth1,'FaceColor','red','EdgeColor','none','FaceAlpha',0.7);
%p2 = patch(ground_truth2,'FaceColor','blue','EdgeColor','none','FaceAlpha',0.3);
%view(3), axis vis3d equal, camlight right, lighting phong