function [cost,ground_truth] = citrus_press_surface
% makes a test volume containing one closed, roughly spherical, surface

dim = [200 200 200];

[X,Y,Z] = ndgrid(1:dim(1),1:dim(2),1:dim(3));

% making rotation matrix
u = [1 2 0];
u = u/norm(u); % rotation axis
alpha = pi/3; % rotation angle
U = zeros(3); % cross product matrix
U([2 6 7])  = u([3 1 2]);
U([3 4 8])  = -u([2 3 1]);
R = eye(3)*cos(alpha) + sin(alpha)*U + (1-cos(alpha))*kron(u',u);
c = (dim+1)/2;

% rotating
P = [X(:)-c(1) Y(:)-c(2) Z(:)-c(3)]';
PR = R*P;
X(:) = PR(1,:)+c(1);
Y(:) = PR(2,:)+c(2);
Z(:) = PR(3,:)+c(3);

% making the distance field
phi = atan2(Y-c(2),X-c(1)); % in [-pi,pi]
theta = acos((Z-c(3))./((X-c(1)).^2+(Y-c(2)).^2+(Z-c(3)).^2).^0.5); % in [0,pi]
r = min(dim)/4*... basic radius
    (1+0.2*sin(8*phi-0.1*Z).*... slices
    (1-cos(2*theta))/2); % spiraling
V = (X-c(1)).^2+(Y-c(2)).^2+(Z-c(3)).^2 - r.^2;

% using sigmoid function as a smooth threshold to calculate cost
t = 100; % pertaining to the thickness of low cost layer
s = 2; % steepness of the low cost layer boundary
cost = 1/(t^s/abs(V).^s+1);

% zero surface
% permuting dimensions since isosurface assumes input defined as meshgrid, not ndgrid
ground_truth = isosurface(permute(V,[2 1 3]),0);

% visualization
%p1 = patch(ground_truth,'FaceColor','red','EdgeColor','none');
%view(3), axis vis3d equal, camlight right, lighting phong
%figure, colormap gray, animate_volume(cost,0,'z')


