function [V,cost_in,cost_on,ground_truth] = multi_sphere(nr_spheres)
% makes a test volume containing two or three connected spheres
% nr_speheres is a string '2' or a string '3'

dim = [200 200 200]; % size of the volume
c = (dim+1)/2;

switch nr_spheres
    case '2'
        os = [c 35;c+[60 0 0] 35;]; % x y z r % TWO SPHERE
    case '3'
        os = [c-[20 0 0] 35; c+[20 0 0] 15; c+[60 0 0] 35]; % THREE SPHERE
end

% spehres defining outer envelope
[X,Y,Z] = ndgrid(1:dim(1),1:dim(2),1:dim(3));

% making the distance field
for i = 1:size(os,1);
    OS(:,:,:,i) = 50/(os(i,4))*((X-os(i,1)).^2+(Y-os(i,2)).^2+(Z-os(i,3)).^2 - os(i,4).^2);
end

S = min(OS,[],4); % union of spheres is <0

% SURFACE COST
S_on = abs(S)<50; % binary image
S_on = S_on + (1/8)*(S_on([1 1:end-1],:,:) + S_on([2:end end],:,:) +...
    S_on(:,[1 1:end-1],:) + S_on(:,[2:end end],:) +...
    S_on(:,:,[1 1:end-1]) + S_on(:,:,[2:end end])); % 3d filtering with [1/8 1 1/8]
cost_on = 1-(S_on-min(S_on(:)))/(max(S_on(:))-min(S_on(:))); % normalization

% REGION COST
cost_in(:,:,:,1) = S>=0; 
cost_in(:,:,:,2) = S<0; 

% TEST_VOLUME
V = mask2data(S,[90 190],[20 10],5);

% zero surface
ground_truth = isosurface(permute(S,[2 1 3]),0);

% visualization 
%figure
%p1 = patch(ground_truth,'FaceColor','red','EdgeColor','none');
%view(3), axis vis3d equal, camlight right, lighting phong
%figure
%animate_volume(cost_on,0,'z',100)
%figure
%animate_volume(cost_in(:,:,:,1),0,'z',100)


