function animate_volume_and_contours(V,mesh,p,direction,range)
% mesh can also be a cell of meshes

tic, fprintf('Animate volume and contours ... ')

if nargin<3
    p = 'no';
end

if nargin<4
    direction = 'xyz';
end

if nargin<5
    range_flag = false;
elseif strcmp(range,'nonempty')
    range_flag = 'nonempty';
else
    range_flag = true;
end

if ~iscell(mesh) % to support more than one layer
    mesh = {mesh};
end

for d = 1:length(direction)
    cla, title('Slicing the mesh'), drawnow, hold on, axis xy equal, colormap gray
    if direction(d)=='x'
        Vdir = permute(V,[2 3 1]);
        vertsdir = cellfun(@(m) m.vertices(:,[2 3 1]),mesh,...
            'UniformOutput',false);
        xlab = 'y'; ylab = 'z';
    elseif direction(d)=='y'
        Vdir = permute(V,[1 3 2]);
        vertsdir = cellfun(@(m) m.vertices(:,[1 3 2]),mesh,...
            'UniformOutput',false);
        xlab = 'x'; ylab = 'z';
    else % 'z'
        Vdir = V;
        vertsdir = cellfun(@(m) m.vertices,mesh,'UniformOutput',false);
        xlab = 'x'; ylab = 'y';
    end
    
    K = length(mesh); % number of layers
    if strcmp(range_flag,'nonempty')
        rmin = size(Vdir,3);
        rmax = 1;
        for k=1:K
            rmin = min(rmin,min(vertsdir{k}(:,3)));
            rmax = max(rmax,max(vertsdir{k}(:,3)));
        end
        rmin = max(floor(rmin),1);
        rmax = min(ceil(rmax),size(Vdir,3));
        range = rmin:rmax;
    elseif ~range_flag
        range = 1:size(Vdir,3);
    end
    
    % input to mesh_slicer are planes ax+by+cz+d=0 given as (a,b,c,d)
    S = numel(range);
    slices = [zeros(S,2),-ones(S,1),range(:)]; % plane z = range
    
    contours = cell(K);
    for k=1:K
        meshk.faces = mesh{k}.faces;
        meshk.vertices = vertsdir{k};
        contours{k} = mesh_slicer(meshk,slices);
    end
    
    col = 'rcgmyb';
    for i = 1:numel(range)
        cla % relevant only if the mesh is OUTSIDE the volumen
        imagesc(Vdir(:,:,range(i))') % transpose since I insist on having xyz volume
        for k=1:K
            if ~isempty(contours{k}{i}{1})
                for l = 1:length(contours{k}{i})
                    plot(contours{k}{i}{l}([1:end,1],1),contours{k}{i}{l}([1:end,1],2),col(mod(k-1,6)+1))
                end
            end
        end
        title(['slice ',direction(d),' ',num2str(range(i)),'/',num2str(size(Vdir,3))])
        xlabel(xlab), ylabel(ylab)        
        smart_pause(p)
    end
end

disp(['finished at ',num2str(toc)])

end




